@@ -22,6 +22,7 @@
 #include <zmq.h>
 static_assert(ZMQ_VERSION_MAJOR >= 3,"The minimum required version of libzmq is 3.0.0.");
 #include <zmq.hpp>
+#include <signal.h>
 #include "interface.h"
 
 SEXP get_zmq_version() {
@@ -122,11 +123,23 @@ static void socketFinalizer(SEXP socket_) {
   }
 }
 
-SEXP initContext() {
+static void messageFinalizer(SEXP msg_) {
+  zmq::message_t* msg = reinterpret_cast<zmq::message_t*>(checkExternalPointer(msg_,"zmq::message_t*"));
+  if(msg) {
+    delete msg; // destructor will call zmq_msg_close()
+    R_ClearExternalPtr(msg_);
+  }
+}
+
+SEXP initContext(SEXP threads_) {
+  if(TYPEOF(threads_) != INTSXP) {
+    error("thread number must be an integer.");
+  }
+
   SEXP context_;
   zmq::context_t* context;
   try {
-    context = new zmq::context_t(1);
+    context = new zmq::context_t(*INTEGER(threads_));
   } catch(std::exception& e) {
     REprintf("%s\n",e.what());
     return R_NilValue;
@@ -220,6 +233,9 @@ static short rzmq_build_event_bitmask(SEXP askevents) {
 
 SEXP pollSocket(SEXP sockets_, SEXP events_, SEXP timeout_) {
     SEXP result;
+#ifdef SIGWINCH
+    signal(SIGWINCH, SIG_IGN);
+#endif
     
     if(TYPEOF(timeout_) != INTSXP) {
         error("poll timeout must be an integer.");
@@ -405,6 +421,59 @@ SEXP sendNullMsg(SEXP socket_, SEXP send_more_) {
   return ans;
 }
 
+SEXP initMessage(SEXP data_) {
+  SEXP msg_;
+
+  if(TYPEOF(data_) != RAWSXP) {
+    REprintf("data type must be raw (RAWSXP).\n");
+    UNPROTECT(1);
+    return R_NilValue;
+  }
+
+  zmq::message_t* msg = new zmq::message_t(length(data_));
+  memcpy(msg->data(), RAW(data_), length(data_));
+// no copy below, see first that one copy works
+//  zmq::message_t msg(reinterpret_cast<void*>(data_), length(data_), NULL);
+
+  PROTECT(msg_ = R_MakeExternalPtr(reinterpret_cast<void*>(msg),install("zmq::message_t*"),R_NilValue));
+  R_RegisterCFinalizerEx(msg_, messageFinalizer, TRUE);
+  UNPROTECT(1);
+  return msg_;
+}
+
+SEXP sendMessageObject(SEXP socket_, SEXP msg_, SEXP send_more_) {
+  SEXP ans; PROTECT(ans = allocVector(LGLSXP,1));
+  bool status(false);
+
+  if(TYPEOF(send_more_) != LGLSXP) {
+    REprintf("send.more type must be logical (LGLSXP).\n");
+    return R_NilValue;
+  }
+
+  zmq::message_t* msg = reinterpret_cast<zmq::message_t*>(checkExternalPointer(msg_,"zmq::message_t*"));
+  if(!msg) { REprintf("bad message object.\n");return R_NilValue; }
+
+  zmq::message_t copy;
+  copy.copy(msg);
+
+  zmq::socket_t* socket = reinterpret_cast<zmq::socket_t*>(checkExternalPointer(socket_,"zmq::socket_t*"));
+  if(!socket) { REprintf("bad socket object.\n");return R_NilValue; }
+
+  bool send_more = LOGICAL(send_more_)[0];
+  try {
+    if(send_more) {
+      status = socket->send(copy,ZMQ_SNDMORE);
+    } else {
+      status = socket->send(copy);
+    }
+  } catch(std::exception& e) {
+    REprintf("%s\n",e.what());
+  }
+  LOGICAL(ans)[0] = static_cast<int>(status);
+  UNPROTECT(1);
+  return ans;
+}
+
 SEXP receiveNullMsg(SEXP socket_) {
   SEXP ans; PROTECT(ans = allocVector(LGLSXP,1));
   bool status(false);
@@ -425,6 +494,9 @@ SEXP receiveNullMsg(SEXP socket_) {
 SEXP receiveSocket(SEXP socket_, SEXP dont_wait_) {
   SEXP ans;
   zmq::message_t msg;
+#ifdef SIGWINCH
+  signal(SIGWINCH, SIG_IGN);
+#endif
 
   if(TYPEOF(dont_wait_) != LGLSXP) {
     REprintf("dont_wait type must be logical (LGLSXP).\n");
